@@ -25,11 +25,12 @@
  * http://www.steadystate.com/css/
  * mailto:css@steadystate.co.uk
  *
- * $Id: CSSValueImpl.java,v 1.6 2008/02/24 13:37:09 xamjadmin Exp $
+ * $Id: CSSValueImpl.java,v 1.7 2008/03/30 21:42:11 xamjadmin Exp $
  */
 
 package com.steadystate.css.dom;
 
+import java.util.logging.*;
 import java.io.IOException;
 import java.io.Serializable;
 import java.io.StringReader;
@@ -53,7 +54,7 @@ import com.steadystate.css.parser.LexicalUnitImpl;
  * @version $Release$
  */
 public class CSSValueImpl implements CSSPrimitiveValue, CSSValueList, Serializable {
-
+	private static final Logger logger = Logger.getLogger(CSSValueImpl.class.getName());
     private Object _value = null;
 
     /**
@@ -73,10 +74,9 @@ public class CSSValueImpl implements CSSPrimitiveValue, CSSValueList, Serializab
                 	v.add(_value);
                 	while(nextValue != null) {
                         if ((nextValue.getLexicalUnitType() != LexicalUnit.SAC_OPERATOR_COMMA)
-                                && (nextValue.getLexicalUnitType() != LexicalUnit.SAC_OPERATOR_SLASH)) {
-                                v.addElement(new CSSValueImpl(nextValue, true));
-                            }
-                		v.add(nextValue);
+                           && (nextValue.getLexicalUnitType() != LexicalUnit.SAC_OPERATOR_SLASH)) {
+                           v.addElement(new CSSValueImpl(nextValue, true));
+                        }
                 		nextValue = nextValue.getNextLexicalUnit();
                 	}
                 	_value = v;
@@ -152,6 +152,18 @@ public class CSSValueImpl implements CSSPrimitiveValue, CSSValueList, Serializab
             				}
             			}
             		}
+            		else if(itemValue instanceof RGBColor) {
+                		if(firstTime) {
+                			firstTime = false;
+                		}
+                		else {
+                			sb.append(" ");
+                		}
+                		sb.append(itemValue.toString());
+            		}
+            		else if(itemValue != null) {
+            			logger.warning("toString(): Unknown item value type: " + itemValue.getClass().getName());
+            		}
             	}
             	else {
             		if(element != null) {
