@@ -33,7 +33,11 @@ import org.w3c.dom.html2.*;
  * it is not designed to know how to perform on its own, e.g. opening a browser window
  * or a context menu. In many ways this
  * interface parallers the Javascript <code>Window</code> class (which
- * in reality represents a browser frame, not a window). 
+ * in reality represents a browser frame, not a window).
+ * <p>
+ * A simple implementation of this interface is provided in
+ * {@link org.lobobrowser.html.test.SimpleHtmlRendererContext SimpleHtmlRendererContext}. 
+ * @see org.lobobrowser.html.gui.HtmlPanel#setDocument(org.w3c.dom.Document, HtmlRendererContext)
  */
 public interface HtmlRendererContext {	
 	/**
@@ -96,8 +100,28 @@ public interface HtmlRendererContext {
 	 * This method is called when a visual element is right-clicked.
 	 * @param element The narrowest element enclosing the mouse location.
 	 * @param event The mouse event.
+	 * @return The method should return true to continue propagating the event,
+	 *         or false to stop propagating it.
 	 */
-	public void onContextMenu(org.w3c.dom.html2.HTMLElement element, java.awt.event.MouseEvent event);
+	public boolean onContextMenu(org.w3c.dom.html2.HTMLElement element, java.awt.event.MouseEvent event);
+
+	/**
+	 * This method is called when there's a mouse click on an element.
+	 * @param element The narrowest element enclosing the mouse location.
+	 * @param event The mouse event.
+	 * @return The method should return true to continue propagating the event,
+	 *         or false to stop propagating it.
+	 */	
+	public boolean onMouseClick(org.w3c.dom.html2.HTMLElement element, java.awt.event.MouseEvent event);
+
+	/**
+	 * This method is called when there's a mouse double-click on an element.
+	 * @param element The narrowest element enclosing the mouse location.
+	 * @param event The mouse event.
+	 * @return The method should return true to continue propagating the event,
+	 *         or false to stop propagating it.
+	 */	
+	public boolean onDoubleClick(org.w3c.dom.html2.HTMLElement element, java.awt.event.MouseEvent event);
 
 	/**
 	 * This method is called when the mouse first hovers over an element.
@@ -113,6 +137,12 @@ public interface HtmlRendererContext {
 	 */
 	public void onMouseOut(org.w3c.dom.html2.HTMLElement element, java.awt.event.MouseEvent event);
 
+	/**
+	 * This method should return true if and only if image loading
+	 * needs to be enabled.
+	 */
+	public boolean isImageLoadingEnabled();
+	
 	//------ Methods useful for Window implementation:
 	
 	/**
