@@ -3,6 +3,8 @@ package org.lobobrowser.html;
 /**
  * Provides information about the user agent (browser) driving
  * the parser and/or renderer.
+ * @see HtmlRendererContext#getUserAgentContext()
+ * @see org.lobobrowser.html.parser.DocumentBuilderImpl#DocumentBuilderImpl(UserAgentContext)
  */
 public interface UserAgentContext {
 	/**
@@ -52,6 +54,12 @@ public interface UserAgentContext {
 	 */
 	public boolean isScriptingEnabled();
 	
+	/**
+	 * Returns a boolean value indicating whether
+	 * remote (non-inline) CSS documents should be loaded.
+	 */
+	public boolean isExternalCSSEnabled();
+	
 	/**
 	 * Gets the name of the user's operating system.
 	 */
