@@ -105,6 +105,25 @@ public class HTMLDocumentImpl extends NodeImpl implements HTMLDocument, Document
 		this.setUserData(Executor.SCOPE_KEY, window.getWindowScope(), null);
 	}
 
+	private Set locales;
+
+	/**
+	 * Gets an <i>immutable</i> set of locales previously set for this document.
+	 */
+	public Set getLocales() {
+		return locales;
+	}
+
+	/**
+	 * Sets the locales of the document. This helps
+	 * determine whether specific fonts can display text
+	 * in the languages of all the locales.
+	 * @param locales An <i>immutable</i> set of <code>java.util.Locale</code> instances.
+	 */
+	public void setLocales(Set locales) {
+		this.locales = locales;
+	}
+
 	String getDocumentHost() {
 		URL docUrl = this.documentURL;
 		return docUrl == null ? null : docUrl.getHost();
@@ -1050,7 +1069,7 @@ public class HTMLDocumentImpl extends NodeImpl implements HTMLDocument, Document
 	 */
 	protected void loadImage(String relativeUri, ImageListener imageListener) {
 		HtmlRendererContext rcontext = this.getHtmlRendererContext();
-		if(rcontext == null) {
+		if(rcontext == null || !rcontext.isImageLoadingEnabled()) {
 			// Ignore image loading when there's no renderer context.
 			// Consider Cobra users who are only using the parser.
 			imageListener.imageLoaded(BLANK_IMAGE_EVENT);
