@@ -472,12 +472,6 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 		super.informInvalid();
 	}
 
-	public void informLayoutInvalid() {
-		// This is called by the style properties object
-		// when certain properties change.
-		super.informLayoutInvalid();
-	}
-
 	/**
 	 * Gets form input due to the current element. It should
 	 * return <code>null</code> except when the element is a form input element.
