@@ -25,7 +25,7 @@ import java.net.URL;
 import org.w3c.dom.UserDataHandler;
 import org.w3c.dom.css.CSSStyleSheet;
 import org.w3c.dom.html2.*;
-import org.lobobrowser.html.HtmlRendererContext;
+import org.lobobrowser.html.*;
 import org.lobobrowser.html.style.*;
 import org.lobobrowser.util.gui.*;
 
@@ -125,29 +125,32 @@ public class HTMLLinkElementImpl extends HTMLAbstractUIElement implements
 	 */
 	protected void processLink() {
 		String rel = this.getAttribute("rel");
-		if(rel != null && rel.toLowerCase().indexOf("stylesheet") != -1) {	
-			String media = this.getMedia();
-			if(CSSUtilities.matchesMedia(media, this.getUserAgentContext())) {
-				HTMLDocumentImpl doc = (HTMLDocumentImpl) this.getOwnerDocument();
-				try {
-					boolean liflag = loggableInfo;
-					long time1 = liflag ? System.currentTimeMillis() : 0;
+		if(rel != null && rel.toLowerCase().indexOf("stylesheet") != -1) {
+			UserAgentContext uacontext = this.getUserAgentContext();
+			if(uacontext.isExternalCSSEnabled()) {
+				String media = this.getMedia();
+				if(CSSUtilities.matchesMedia(media, uacontext)) {
+					HTMLDocumentImpl doc = (HTMLDocumentImpl) this.getOwnerDocument();
 					try {
-						CSSStyleSheet sheet = CSSUtilities.parse(this, this.getHref(), doc, doc.getBaseURI(), false);
-						if(sheet != null) {
-							doc.addStyleSheet(sheet);
+						boolean liflag = loggableInfo;
+						long time1 = liflag ? System.currentTimeMillis() : 0;
+						try {
+							CSSStyleSheet sheet = CSSUtilities.parse(this, this.getHref(), doc, doc.getBaseURI(), false);
+							if(sheet != null) {
+								doc.addStyleSheet(sheet);
+							}
+						} finally {
+							if(liflag) {
+								long time2 = System.currentTimeMillis();
+								logger.info("processLink(): Loaded and parsed CSS (or attempted to) at URI=[" + this.getHref() + "] in " + (time2 - time1) + " ms.");
+							}
 						}
-					} finally {
-						if(liflag) {
-							long time2 = System.currentTimeMillis();
-							logger.info("processLink(): Loaded and parsed CSS (or attempted to) at URI=[" + this.getHref() + "] in " + (time2 - time1) + " ms.");
-						}
-					}
 
-				} catch(MalformedURLException mfe) {
-					this.warn("Will not parse CSS. URI=[" + this.getHref() + "] with BaseURI=[" + doc.getBaseURI() + "] does not appear to be a valid URI.");
-				} catch(Throwable err) {
-					this.warn("Unable to parse CSS. URI=[" + this.getHref() + "].", err);
+					} catch(MalformedURLException mfe) {
+						this.warn("Will not parse CSS. URI=[" + this.getHref() + "] with BaseURI=[" + doc.getBaseURI() + "] does not appear to be a valid URI.");
+					} catch(Throwable err) {
+						this.warn("Unable to parse CSS. URI=[" + this.getHref() + "].", err);
+					}
 				}
 			}
 		}
