@@ -35,14 +35,7 @@ public class HTMLOptionElementImpl extends HTMLElementImpl implements HTMLOption
 	}
 
 	public boolean getSelected() {
-		Object parent = this.getParentNode();
-		if(parent instanceof HTMLSelectElement) {
-			int selectedIndex = ((HTMLSelectElement) parent).getSelectedIndex();
-			return selectedIndex != -1 && selectedIndex == this.getIndex();
-		} 
-		else {
-			return false;
-		}		
+		return this.selected;
 	}
 
 	public String getText() {
@@ -65,17 +58,28 @@ public class HTMLOptionElementImpl extends HTMLElementImpl implements HTMLOption
 		this.setAttribute("label", label);
 	}
 
+	private boolean selected;
+
+	void setSelectedImpl(boolean selected) {
+		this.selected = selected;
+	}
+
 	public void setSelected(boolean selected) {
+		boolean changed = selected != this.selected;
+		this.selected = selected;
+		// Changing the option state changes the selected index.
 		Object parent = this.getParentNode();
-		if(parent instanceof HTMLSelectElement) {
-			if(selected) {	
-				((HTMLSelectElement) parent).setSelectedIndex(this.getIndex());				
-			}
-			else {
-				HTMLSelectElement parentSelect = ((HTMLSelectElement) parent);
-				int currentIndex = parentSelect.getSelectedIndex();
-				if(currentIndex != -1 && currentIndex == this.getIndex()) {
-					parentSelect.setSelectedIndex(-1);
+		if(parent instanceof HTMLSelectElementImpl) {
+			HTMLSelectElementImpl parentSelect = ((HTMLSelectElementImpl) parent);
+			if(changed || parentSelect.getSelectedIndex() == -1) {
+				if(selected) {	
+					parentSelect.setSelectedIndexImpl(this.getIndex());				
+				}
+				else {
+					int currentIndex = parentSelect.getSelectedIndex();
+					if(currentIndex != -1 && currentIndex == this.getIndex()) {
+						parentSelect.setSelectedIndexImpl(-1);
+					}
 				}
 			}
 		}
