@@ -1,6 +1,8 @@
 package org.lobobrowser.html.domimpl;
 
+import java.util.*;
 import org.lobobrowser.html.FormInput;
+import org.mozilla.javascript.*;
 import org.w3c.dom.DOMException;
 import org.w3c.dom.html2.HTMLElement;
 import org.w3c.dom.html2.HTMLOptionsCollection;
@@ -21,14 +23,14 @@ public class HTMLSelectElementImpl extends HTMLBaseInputElement implements
 		return this.getOptions().getLength();
 	}
 
+	private Boolean multipleState = null;
+	
 	public boolean getMultiple() {
-		InputContext ic = this.inputContext;
-		if(ic != null) {
-			return ic.getMultiple();
-		}
-		else {
-			return false;
+		Boolean m = this.multipleState;
+		if(m != null) {
+			return m.booleanValue();
 		}
+		return this.getAttributeAsBoolean("multiple");
 	}
 
 	private HTMLOptionsCollection options;
@@ -79,15 +81,26 @@ public class HTMLSelectElementImpl extends HTMLBaseInputElement implements
 	}
 
 	public void setMultiple(boolean multiple) {
-		InputContext ic = this.inputContext;
-		if(ic != null) {
-			ic.setMultiple(multiple);
+		boolean prevMultiple = this.getMultiple();
+		this.multipleState = Boolean.valueOf(multiple);
+		if(prevMultiple != multiple) {
+			this.informLayoutInvalid();
 		}
 	}
 
 	private int deferredSelectedIndex = -1;
 	
 	public void setSelectedIndex(int selectedIndex) {
+		this.setSelectedIndexImpl(selectedIndex);
+		HTMLOptionsCollection options = this.getOptions();
+		int length = options.getLength();
+		for(int i = 0; i < length; i++) {
+			HTMLOptionElementImpl option = (HTMLOptionElementImpl) options.item(i);
+			option.setSelectedImpl(i == selectedIndex);
+		}
+	}
+
+	void setSelectedIndexImpl(int selectedIndex) {
 		InputContext ic = this.inputContext;
 		if(ic != null) {
 			ic.setSelectedIndex(selectedIndex);
@@ -106,11 +119,24 @@ public class HTMLSelectElementImpl extends HTMLBaseInputElement implements
 
 	protected FormInput[] getFormInputs() {
 		// Needs to be overriden for forms to submit.
+		InputContext ic = this.inputContext;
+		String[] values = ic == null ? null : ic.getValues();
+		if(values == null) {
+			String value = this.getValue();
+			values = value == null ? null : new String[] { value };
+			if(values == null) {
+				return null;
+			}
+		}
 		String name = this.getName();
 		if(name == null) {
 			return null;
 		}
-		return new FormInput[] { new FormInput(name, this.getValue()) };
+		ArrayList formInputs = new ArrayList();
+		for(int i = 0; i < values.length; i++) {
+			formInputs.add(new FormInput(name, values[i]));
+		}
+		return (FormInput[]) formInputs.toArray(FormInput.EMPTY_ARRAY);
 	}	
 	
 	public void resetInput() {
@@ -126,4 +152,14 @@ public class HTMLSelectElementImpl extends HTMLBaseInputElement implements
 			ic.setSelectedIndex(this.deferredSelectedIndex);
 		}
 	}
+	
+	private Function onchange;
+	
+	public Function getOnchange() {
+		return this.getEventFunction(this.onchange, "onchange");
+	}
+	
+	public void setOnchange(Function value) {
+		this.onchange = value;
+	}
 }
