@@ -26,6 +26,7 @@ package org.lobobrowser.html.domimpl;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.util.*;
+import java.util.Map.Entry;
 import java.util.logging.*;
 
 import org.lobobrowser.html.*;
@@ -230,6 +231,17 @@ public abstract class NodeImpl extends AbstractScriptableDelegate implements Nod
 					}
 				}
 			}
+			
+			synchronized(this){
+				if (userDataHandlers != null && userData != null) {
+					for(Iterator handlers = userDataHandlers.entrySet().iterator();handlers.hasNext();){
+						Map.Entry entry = (Map.Entry) handlers.next();    				
+						UserDataHandler handler = (UserDataHandler)entry.getValue();     				
+						handler.handle(UserDataHandler.NODE_CLONED,(String) entry.getKey(),userData.get(entry.getKey()), this, newNode);    				     				
+					}
+				}
+			}
+						
 			return newNode;
 		} catch(Exception err) {
 			throw new IllegalStateException(err.getMessage());
@@ -539,7 +551,7 @@ public abstract class NodeImpl extends AbstractScriptableDelegate implements Nod
 
 	private Map userData;
 	//TODO: Inform handlers on cloning, etc.
-	private List userDataHandlers;
+	private Map userDataHandlers;
 	protected volatile boolean notificationsSuspended = false;
 	
 	public Object setUserData(String key, Object data,
@@ -551,19 +563,31 @@ public abstract class NodeImpl extends AbstractScriptableDelegate implements Nod
 				this.informNodeLoaded();
 			}
 		}
+		//here we spent some effort preventing our maps from growing too much
 		synchronized(this) {
 			if(handler != null) {
 				if(this.userDataHandlers == null) {
-					this.userDataHandlers = new LinkedList();
+					this.userDataHandlers = new HashMap();
+				}	
+				else if (handler == null){
+					this.userDataHandlers.remove(key);
 				}
-				this.userDataHandlers.add(handler);
+				if (handler != null)
+					this.userDataHandlers.put(key,handler);
 			}
+						
 			Map userData = this.userData;
-			if(userData == null) {
-				userData = new HashMap();
-				this.userData = userData;
+			if (data != null){
+				if(userData == null) {
+					userData = new HashMap();
+					this.userData = userData;
+				}
+				return userData.put(key, data);
 			}
-			return userData.put(key, data);
+			else if (userData != null)
+				return userData.remove(key);
+			else
+				return null;
 		}
 	}
 
