@@ -786,8 +786,10 @@ public class HtmlBlockPanel extends JComponent implements NodeRenderer, Renderab
 		// them to lose focus.
 		
 		Set c = this.components;
-		if(c == null && this.getComponentCount() != 0) {
-			this.removeAll();
+		if(c == null) {
+			if(this.getComponentCount() != 0) {
+				this.removeAll();
+			}
 		}
 		else {
 			// Remove children not in the set.
