@@ -51,15 +51,17 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	private final EventDispatch2 selectionDispatch = new SelectionDispatch();
 	private final javax.swing.Timer notificationTimer;
 	private final DocumentNotificationListener notificationListener;
+	private final Runnable notificationImmediateAction;
 	private static final int NOTIF_TIMER_DELAY = 300;
 
 	private volatile boolean isFrameSet = false;
 	private volatile NodeRenderer nodeRenderer = null;
 	private volatile NodeImpl rootNode;
-	private volatile HtmlBlockPanel htmlBlock;
-	private volatile FrameSetPanel frameSetPanel;
 	private volatile int preferredWidth = -1;
-	
+
+	protected volatile HtmlBlockPanel htmlBlockPanel;
+	protected volatile FrameSetPanel frameSetPanel;
+
 	/**
 	 * Constructs an <code>HtmlPanel</code>.
 	 */
@@ -70,6 +72,11 @@ public class HtmlPanel extends JComponent implements FrameContext {
 		this.notificationTimer = new javax.swing.Timer(NOTIF_TIMER_DELAY, new NotificationTimerAction());
 		this.notificationTimer.setRepeats(false);
 		this.notificationListener = new LocalDocumentNotificationListener();
+		this.notificationImmediateAction = new Runnable() {
+			public void run() {
+				processNotifications();
+			}
+		};
 	}
 
 	/**
@@ -88,7 +95,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 */
 	public void setPreferredWidth(int width) {
 		this.preferredWidth = width;
-		HtmlBlockPanel htmlBlock = this.htmlBlock;
+		HtmlBlockPanel htmlBlock = this.htmlBlockPanel;
 		if(htmlBlock != null) {
 			htmlBlock.setPreferredWidth(width);
 		}
@@ -107,7 +114,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 *                  requested bounds are not currently visible vertically.
 	 */
 	public void scrollTo(Rectangle bounds, boolean xIfNeeded, boolean yIfNeeded) {
-		HtmlBlockPanel htmlBlock = this.htmlBlock;
+		HtmlBlockPanel htmlBlock = this.htmlBlockPanel;
 		if(htmlBlock != null) {
 			htmlBlock.scrollTo(bounds, xIfNeeded, yIfNeeded);
 		}		
@@ -121,7 +128,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * @param node A DOM node.
 	 */
 	public void scrollTo(org.w3c.dom.Node node) {
-		HtmlBlockPanel htmlBlock = this.htmlBlock;
+		HtmlBlockPanel htmlBlock = this.htmlBlockPanel;
 		if(htmlBlock != null) {
 			htmlBlock.scrollTo(node);
 		}				
@@ -133,7 +140,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * for FRAMESETs.
 	 */
 	public BoundableRenderable getBlockRenderable() {
-		HtmlBlockPanel htmlBlock = this.htmlBlock;
+		HtmlBlockPanel htmlBlock = this.htmlBlockPanel;
 		return htmlBlock == null ? null : htmlBlock.getRootRenderable();
 	}
 	
@@ -160,7 +167,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	private void setUpAsBlock(UserAgentContext ucontext, HtmlRendererContext rcontext) {
 		HtmlBlockPanel shp = this.createHtmlBlockPanel(ucontext, rcontext);
 		shp.setPreferredWidth(this.preferredWidth);
-		this.htmlBlock = shp;
+		this.htmlBlockPanel = shp;
 		this.frameSetPanel = null;
 		shp.setDefaultPaddingInsets(new Insets(8, 8, 8, 8));
 		this.removeAll();
@@ -170,7 +177,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 
 	private void setUpFrameSet(NodeImpl fsrn) {
 		this.isFrameSet = true;
-		this.htmlBlock = null;
+		this.htmlBlockPanel = null;
 		FrameSetPanel fsp = this.createFrameSetPanel();
 		this.frameSetPanel = fsp;
 		this.nodeRenderer = fsp;
@@ -249,7 +256,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 			nr.setRootNode(null);
 		}
 		this.rootNode = null;
-		this.htmlBlock = null;
+		this.htmlBlockPanel = null;
 		this.nodeRenderer = null;
 		this.isFrameSet = false;
 		this.removeAll();
@@ -271,6 +278,8 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * {@link org.lobobrowser.html.parser.DocumentBuilderImpl}.
 	 * <p>
 	 * @param rcontext A renderer context.
+	 * @see org.lobobrowser.html.parser.DocumentBuilderImpl#parse(org.xml.sax.InputSource)
+	 * @see org.lobobrowser.html.test.SimpleHtmlRendererContext
 	 */
 	public void setDocument(final Document node, final HtmlRendererContext rcontext) {
 		if(java.awt.EventQueue.isDispatchThread()) {
@@ -364,6 +373,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * @param htmlSource The HTML source code.
 	 * @param uri A base URI used to resolve item URIs.
 	 * @param rcontext The {@link HtmlRendererContext} instance.
+	 * @see org.lobobrowser.html.test.SimpleHtmlRendererContext
 	 */
 	public void setHtml(String htmlSource, String uri, HtmlRendererContext rcontext) {
 		try {
@@ -492,7 +502,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * Note: This method should be invoked in the GUI thread.
 	 */
 	public void expandSelection(RenderableSpot rpoint) {
-		HtmlBlockPanel block = this.htmlBlock;
+		HtmlBlockPanel block = this.htmlBlockPanel;
 		if(block != null) {
 			block.setSelectionEnd(rpoint);
 			block.repaint();
@@ -508,7 +518,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * Note: This method should be invoked in the GUI thread.
 	 */
 	public void resetSelection(RenderableSpot rpoint) {
-		HtmlBlockPanel block = this.htmlBlock;
+		HtmlBlockPanel block = this.htmlBlockPanel;
 		if(block != null) {
 			block.setSelectionStart(rpoint);
 			block.setSelectionEnd(rpoint);
@@ -523,7 +533,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * Note: This method should be invoked in the GUI thread.
 	 */
 	public String getSelectionText() {
-		HtmlBlockPanel block = this.htmlBlock;
+		HtmlBlockPanel block = this.htmlBlockPanel;
 		if(block == null) {
 			return null;
 		}
@@ -544,7 +554,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 *         for FRAMESETs.
 	 */
 	public org.w3c.dom.Node getSelectionNode() {
-		HtmlBlockPanel block = this.htmlBlock;
+		HtmlBlockPanel block = this.htmlBlockPanel;
 		if(block == null) {
 			return null;
 		}
@@ -558,7 +568,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * This method has no effect in FRAMESETs at the moment.
 	 */
 	public boolean hasSelection() {
-		HtmlBlockPanel block = this.htmlBlock;
+		HtmlBlockPanel block = this.htmlBlockPanel;
 		if(block == null) {
 			return false;
 		}
@@ -572,7 +582,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * This method has no effect in FRAMESETs at the moment.
 	 */
 	public boolean copy() {
-		HtmlBlockPanel block = this.htmlBlock;
+		HtmlBlockPanel block = this.htmlBlockPanel;
 		if(block != null) {
 			return block.copy();
 		}			
@@ -607,8 +617,12 @@ public class HtmlPanel extends JComponent implements FrameContext {
 			notifs.add(notification);
 		}
 		if(EventQueue.isDispatchThread()) {
-			// Process imediately
-			this.processNotifications();
+			// In this case we want the notification to be processed
+			// immediately. However, we don't want potential recursions
+			// to occur when a Javascript property is set in the GUI thread.
+			// Additionally, many property values may be set in one
+			// event block.
+			EventQueue.invokeLater(this.notificationImmediateAction);
 		}
 		else {
 			this.notificationTimer.restart();
@@ -644,14 +658,14 @@ public class HtmlPanel extends JComponent implements FrameContext {
 		int length = notifsArray.length;
 		for(int i = 0; i < length; i++) {
 			DocumentNotification dn = notifsArray[i];
-			if(dn.node instanceof HTMLFrameSetElement && this.htmlBlock != null) {
+			if(dn.node instanceof HTMLFrameSetElement && this.htmlBlockPanel != null) {
 				if(this.resetIfFrameSet()) {
 					// Revalidation already taken care of.
 					return;
 				}
 			}
 		}
-		HtmlBlockPanel blockPanel = this.htmlBlock;
+		HtmlBlockPanel blockPanel = this.htmlBlockPanel;
 		if(blockPanel != null) {
 			blockPanel.processDocumentNotifications(notifsArray);
 		}
