@@ -163,6 +163,96 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
 //		return null;
 //	}
 	
+	
+     /**
+      * @param aFunction
+      *            Javascript function to invoke on each loop.
+      * @param aTimeInMs
+      *            Time in millisecund between each loop.
+      * @return Return the timer ID to use as reference
+      * @see <a href="http://developer.mozilla.org/en/docs/DOM:window.setInterval">Window.setInterval interface
+      *      definition</a>
+      * @todo Make proper and refactore with {@link Window#setTimeout(Function, double)}.
+      */
+     public int setInterval(final Function aFunction, double aTimeInMs)
+     {
+         final int timeID = ID.generateInt();
+         final Integer timeIDInt = new Integer(timeID);
+         ActionListener task = new ActionListener() {
+             public void actionPerformed(ActionEvent e)
+             {
+                 // This executes in the GUI thread and that's good.
+                 Window.this.forgetTask(timeIDInt, false);
+                 HTMLDocumentImpl doc = (HTMLDocumentImpl) document;
+                 if (doc == null)
+                 {
+                     //  FIXME : Make a proper method to stop all timer launched via setInterval, other than window.setDocument(null)
+                     throw new IllegalStateException("Cannot perform operation when document is unset.");
+                 }
+                 Executor.executeFunction(getWindowScope(), aFunction, doc.getDocumentURL(), uaContext);
+             }
+         };
+         if (aTimeInMs > Integer.MAX_VALUE || aTimeInMs < 0)
+         {
+             throw new IllegalArgumentException("Timeout value " + aTimeInMs + " is not supported.");
+         }
+         Timer timer = new Timer((int) aTimeInMs, task);
+         timer.setRepeats(true); // The only difference with setTimeout
+         timer.start();
+         this.putTask(timeIDInt, timer);
+         return timeID;
+     }
+ 
+     /**
+      * @param aExpression
+      *            Javascript expression to invoke on each loop.
+      * @param aTimeInMs
+      *            Time in millisecund between each loop.
+      * @return Return the timer ID to use as reference
+      * @see <a href="http://developer.mozilla.org/en/docs/DOM:window.setInterval">Window.setInterval interface
+      *      definition</a>
+      * @todo Make proper and refactore with {@link Window#setTimeout(String, double)}.
+      */
+     public int setInterval(final String aExpression, double aTimeInMs)
+     {
+    	 final int timeID = ID.generateInt();
+    	 final Integer timeIDInt = new Integer(timeID);
+    	 ActionListener task = new ActionListener() {
+    		 public void actionPerformed(ActionEvent e)
+    		 {
+    			 // This executes in the GUI thread and that's good.
+    			 Window.this.forgetTask(timeIDInt, false);
+    			 HTMLDocumentImpl doc = (HTMLDocumentImpl) document;
+    			 if (doc == null)
+    			 {
+    				 //  FIXME : Make a proper method to stop all timer launched via setInterval, other than window.setDocument(null)
+    				 throw new IllegalStateException("Cannot perform operation when document is unset.");
+    			 }
+    			 Window.this.eval(aExpression);
+    		 }
+    	 };
+    	 if (aTimeInMs > Integer.MAX_VALUE || aTimeInMs < 0)
+    	 {
+    		 throw new IllegalArgumentException("Timeout value " + aTimeInMs + " is not supported.");
+    	 }
+    	 Timer timer = new Timer((int) aTimeInMs, task);
+    	 timer.setRepeats(false); // The only difference with setTimeout
+    	 timer.start();
+    	 this.putTask(timeIDInt, timer);
+    	 return timeID;
+     }
+
+     /**
+      * @param aTimerID
+      *          Timer ID to stop.
+      * @see <a href="http://developer.mozilla.org/en/docs/DOM:window.clearInterval">Window.clearInterval interface Definition</a>
+      */
+     public void clearInterval(int aTimerID)
+     {
+    	 Integer key = new Integer(aTimerID);
+    	 this.forgetTask(key, true);
+     }
+
 	public void alert(String message) {
 		if(this.rcontext != null) {
 			this.rcontext.alert(message);
