@@ -103,10 +103,12 @@ public class DocumentBuilderImpl extends DocumentBuilder {
 	 * Creates a document without parsing it so
 	 * it can be used for incremental rendering.
 	 * @param is The input source, which may be an instance of
-	 * {@link org.lobobrowser.html.parser.InputSourceImpl}.
+	 * {@link org.lobobrowser.html.parser.InputSourceImpl}. The input
+	 * source must provide either an input stream or a reader.
 	 */
 	public Document createDocument(InputSource is) throws SAXException, IOException  {
-		String charset = is.getEncoding();
+		String encoding = is.getEncoding();
+		String charset = encoding;
 		if(charset == null) {
 			charset = "US-ASCII";
 		}
@@ -114,26 +116,33 @@ public class DocumentBuilderImpl extends DocumentBuilder {
 		if(uri == null) {
 			logger.warning("parse(): InputSource has no SystemId (URI); document item URLs will not be resolvable.");
 		}
-		InputStream in = is.getByteStream();
 		WritableLineReader wis;
-		if(in != null) {
-			wis = new WritableLineReader(new InputStreamReader(in, charset));
+		Reader reader = is.getCharacterStream();
+		if(reader != null) {
+			wis = new WritableLineReader(reader);
 		}
 		else {
-			Reader reader = is.getCharacterStream();
-			if(reader != null) {
-				wis = new WritableLineReader(reader);
+			InputStream in = is.getByteStream();
+			if(in != null) {
+				wis = new WritableLineReader(new InputStreamReader(in, charset));
 			}
-			else {
-				throw new IllegalArgumentException("InputSource has neither a byte stream nor a character stream");
+			else if(uri != null) {
+				// To comply with the InputSource documentation, we need
+				// to do this:
+				java.net.URLConnection connection = new java.net.URL(uri).openConnection(); 
+				in = connection.getInputStream();
+				if(encoding == null) {
+					charset = org.lobobrowser.util.Urls.getCharset(connection);
+				}
+				wis = new WritableLineReader(new InputStreamReader(in, charset));
+			} else {
+				throw new IllegalArgumentException("The InputSource must have either a reader, an input stream or a URI.");
 			}
 		}
 		HTMLDocumentImpl document = new HTMLDocumentImpl(this.bcontext, this.rcontext, wis, uri);
 		return document;
 	}
 	
-	//TODO: parseAsync
-
 	public boolean isNamespaceAware() {
 		return false;
 	}
