@@ -25,17 +25,17 @@ package org.lobobrowser.html.parser;
 
 import java.io.*;
 import java.util.*;
-import org.w3c.dom.html2.*;
+import java.util.logging.*;
 import org.w3c.dom.Node;
-import org.w3c.dom.Element;
 import org.w3c.dom.Document;
+import org.w3c.dom.Element;
+import org.w3c.dom.DOMException;
 import org.xml.sax.ErrorHandler;
 import org.xml.sax.Locator;
 import org.xml.sax.SAXException;
 import org.xml.sax.SAXParseException;
 import org.lobobrowser.html.*;
 import org.lobobrowser.html.io.*;
-import org.mozilla.javascript.*;
 
 /**
  * The <code>HtmlParser</code> class is an HTML DOM parser.
@@ -45,7 +45,8 @@ import org.mozilla.javascript.*;
  * implementation is preferred.
  */
 public class HtmlParser {
-	private final HTMLDocument document;
+	private static final Logger logger = Logger.getLogger(HtmlParser.class.getName());
+	private final Document document;
 	private final UserAgentContext ucontext;
 	private final ErrorHandler errorHandler;
 	private final String publicId;
@@ -72,8 +73,8 @@ public class HtmlParser {
 		entities.put("quot", new Character('"'));
 		entities.put("nbsp", new Character((char) 160));
 
-		entities.put("lsquo", new Character('`'));
-		entities.put("rsquo", new Character(''));
+		entities.put("lsquo", new Character('\u2018'));
+		entities.put("rsquo", new Character('\u2019'));
 
 		entities.put("frasl", new Character((char) 47));
 		entities.put("ndash", new Character((char) 8211));
@@ -350,7 +351,8 @@ public class HtmlParser {
 		
 		ElementInfo optionalEndElement = new ElementInfo(true, ElementInfo.END_ELEMENT_OPTIONAL);
 		ElementInfo forbiddenEndElement = new ElementInfo(false, ElementInfo.END_ELEMENT_FORBIDDEN);
-		ElementInfo onlyText = new ElementInfo(false, ElementInfo.END_ELEMENT_REQUIRED);
+		ElementInfo onlyTextDE = new ElementInfo(false, ElementInfo.END_ELEMENT_REQUIRED, true);
+		ElementInfo onlyText = new ElementInfo(false, ElementInfo.END_ELEMENT_REQUIRED, false);
 		
 		Set tableCellStopElements = new HashSet();
 		tableCellStopElements.add("TH");
@@ -386,7 +388,7 @@ public class HtmlParser {
 		
 		elementInfos.put("SCRIPT", onlyText);
 		elementInfos.put("STYLE", onlyText);
-		elementInfos.put("TEXTAREA", onlyText);
+		elementInfos.put("TEXTAREA", onlyTextDE);
 		elementInfos.put("IMG", forbiddenEndElement);
 		elementInfos.put("META", forbiddenEndElement);
 		elementInfos.put("LINK", forbiddenEndElement);
@@ -412,13 +414,13 @@ public class HtmlParser {
 
 	/**
 	 * Constructs a <code>HtmlParser</code>.
-	 * @param document An instanceof of <code>HTMLDocument</code>.
+	 * @param document A W3C Document instance.
 	 * @param errorHandler The error handler. 
 	 * @param publicId The public ID of the document.
 	 * @param systemId The system ID of the document.
 	 * @deprecated UserAgentContext should be passed in constructor.
 	 */
-	public HtmlParser(HTMLDocument document, ErrorHandler errorHandler, String publicId, String systemId) {
+	public HtmlParser(Document document, ErrorHandler errorHandler, String publicId, String systemId) {
 		this.ucontext = null;
 		this.document = document;
 		this.errorHandler = errorHandler;
@@ -429,12 +431,12 @@ public class HtmlParser {
 	/**
 	 * Constructs a <code>HtmlParser</code>.
 	 * @param ucontext The user agent context.
-	 * @param document An instanceof of <code>HTMLDocument</code>.
+	 * @param document An W3C Document instance.
 	 * @param errorHandler The error handler. 
 	 * @param publicId The public ID of the document.
 	 * @param systemId The system ID of the document.
 	 */
-	public HtmlParser(UserAgentContext ucontext, HTMLDocument document, ErrorHandler errorHandler, String publicId, String systemId) {
+	public HtmlParser(UserAgentContext ucontext, Document document, ErrorHandler errorHandler, String publicId, String systemId) {
 		this.ucontext = ucontext;
 		this.document = document;
 		this.errorHandler = errorHandler;
@@ -445,9 +447,9 @@ public class HtmlParser {
 	/**
 	 * Constructs a <code>HtmlParser</code>.
 	 * @param ucontext The user agent context.
-	 * @param document An instanceof of <code>HTMLDocument</code>.
+	 * @param document A W3C Document instance.
 	 */
-	public HtmlParser(UserAgentContext ucontext, HTMLDocument document) {
+	public HtmlParser(UserAgentContext ucontext, Document document) {
 		this.ucontext = ucontext;
 		this.document = document;
 		this.errorHandler = null;
@@ -567,9 +569,15 @@ public class HtmlParser {
 		}
 		if(textSb.length() != 0) {
 			int textLine = reader.getLineNumber();
-			StringBuffer decText = this.entityDecode(textSb, textLine);
+			StringBuffer decText = this.entityDecode(textSb);
 			Node textNode = doc.createTextNode(decText.toString());
-			parent.appendChild(textNode);
+			try {
+				parent.appendChild(textNode);
+			} catch(DOMException de) {
+				if(parent.getNodeType() != Node.DOCUMENT_NODE || de.code != DOMException.HIERARCHY_REQUEST_ERR) {
+					logger.log(Level.WARNING, "parseToken(): Unable to append child to " + parent + ".", de);
+				}
+			}
 		}
 		if(this.justReadTagBegin) {
 			String tag = this.readTag(reader);
@@ -582,7 +590,7 @@ public class HtmlParser {
 					if("!--".equals(tag)) {
 						int commentLine = reader.getLineNumber();
 						StringBuffer comment = this.passEndOfComment(reader);
-						StringBuffer decText = this.entityDecode(comment, commentLine);
+						StringBuffer decText = this.entityDecode(comment);
 						parent.appendChild(doc.createComment(decText.toString()));
 						return TOKEN_COMMENT;
 					}
@@ -650,14 +658,14 @@ public class HtmlParser {
 											if(einfo != null && einfo.noScriptElement) {
 												UserAgentContext ucontext = this.ucontext;
 												if(ucontext == null || ucontext.isScriptingEnabled()) {
-													token = this.parseForEndTag(parent, reader, tag, false);
+													token = this.parseForEndTag(parent, reader, tag, false, einfo.decodeEntities);
 												}
 												else {
 													token = this.parseToken(element, reader, newStopSet, ancestors);
 												}
 											}
 											else {
-												token = childrenOk ? this.parseToken(element, reader, newStopSet, ancestors) : this.parseForEndTag(element, reader, tag, true);
+												token = childrenOk ? this.parseToken(element, reader, newStopSet, ancestors) : this.parseForEndTag(element, reader, tag, true, einfo.decodeEntities);
 											}
 											if(token == TOKEN_END_ELEMENT) {
 												String normalLastTag = this.normalLastTag;
@@ -786,7 +794,7 @@ public class HtmlParser {
 	 * @return
 	 * @throws IOException
 	 */
-	private final int parseForEndTag(Node parent, LineNumberReader reader, String tagName, boolean addTextNode) throws IOException {		
+	private final int parseForEndTag(Node parent, LineNumberReader reader, String tagName, boolean addTextNode, boolean decodeEntities) throws IOException, SAXException {		
 	    Document doc = this.document;
 	    int intCh;
 	    StringBuffer sb = new StringBuffer();
@@ -809,6 +817,9 @@ public class HtmlParser {
 	    							this.justReadEmptyElement = false;
 	    							this.normalLastTag = thisTag.toUpperCase();
 	    							if(addTextNode) {
+    									if(decodeEntities) {
+    										sb = this.entityDecode(sb);
+    									}
 	    								String text = sb.toString();
 	    								if(text.length() != 0) {
 	    									Node textNode = doc.createTextNode(text);
@@ -839,6 +850,9 @@ public class HtmlParser {
 	    this.justReadTagEnd = false;
 	    this.justReadEmptyElement = false;
 	    if(addTextNode) {
+			if(decodeEntities) {
+				sb = this.entityDecode(sb);
+			}
 	    	String text = sb.toString();
 	    	if(text.length() != 0) {
 	    		Node textNode = doc.createTextNode(text);
@@ -1173,7 +1187,7 @@ public class HtmlParser {
 						element.setAttribute(attributeNameStr, null);
 					}
 					else {
-						StringBuffer actualAttributeValue = this.entityDecode(attributeValue, reader.getLineNumber());
+						StringBuffer actualAttributeValue = this.entityDecode(attributeValue);
 						element.setAttribute(attributeNameStr, actualAttributeValue.toString());
 					}
 				}
@@ -1188,7 +1202,7 @@ public class HtmlParser {
 						element.setAttribute(attributeNameStr, null);
 					}
 					else {
-						StringBuffer actualAttributeValue = this.entityDecode(attributeValue, reader.getLineNumber());
+						StringBuffer actualAttributeValue = this.entityDecode(attributeValue);
 						element.setAttribute(attributeNameStr, actualAttributeValue.toString());
 					}
 				}
@@ -1205,7 +1219,7 @@ public class HtmlParser {
 						element.setAttribute(attributeNameStr, null);
 					}
 					else {
-						StringBuffer actualAttributeValue = this.entityDecode(attributeValue, reader.getLineNumber());
+						StringBuffer actualAttributeValue = this.entityDecode(attributeValue);
 						element.setAttribute(attributeNameStr, actualAttributeValue.toString());
 					}
 				}
@@ -1232,14 +1246,14 @@ public class HtmlParser {
 				element.setAttribute(attributeNameStr, null);
 			}
 			else {
-				StringBuffer actualAttributeValue = this.entityDecode(attributeValue, reader.getLineNumber());
+				StringBuffer actualAttributeValue = this.entityDecode(attributeValue);
 				element.setAttribute(attributeNameStr, actualAttributeValue.toString());
 			}
 		}
 		return false;
 	}
 		
-	private final StringBuffer entityDecode(StringBuffer rawText, int lineNumber) throws org.xml.sax.SAXException {
+	private final StringBuffer entityDecode(StringBuffer rawText) throws org.xml.sax.SAXException {
 		int startIdx = 0;
 		StringBuffer sb = null;
 		for(;;) {
@@ -1275,9 +1289,7 @@ public class HtmlParser {
 						decimal = Integer.parseInt(number);
 					}
 				} catch(NumberFormatException nfe) {
-					if(this.errorHandler != null) {
-						this.errorHandler.error(new SAXParseException("Bad entity: " + spec, this.getLocator(lineNumber, 0)));
-					}
+					logger.log(Level.WARNING, "entityDecode()", nfe);
 					decimal = 0;
 				}
 				sb.append((char) decimal);
