@@ -79,4 +79,8 @@ class BaseRListElement extends RBlock {
 		}
 		this.listStyle = listStyle;
 	}
+	
+	public String toString() {
+		return "BaseRListElement[node=" + this.modelNode + "]";
+	}
 }
