@@ -365,11 +365,6 @@ public class RBlock extends BaseElementRenderable implements
 		this.doLayout(availWidth, availHeight, null, 0, this.defaultOverflow);
 	}
 
-	public final void doLayout(int availWidth, int availHeight, int tentativeY) {
-		this.doLayout(availWidth, availHeight, null, tentativeY,
-				this.defaultOverflow);
-	}
-
 	/**
 	 * Lays out and sets dimensions only if RBlock is invalid (or never before
 	 * layed out), if the parameters passed differ from the last layout, or if
@@ -1387,15 +1382,22 @@ public class RBlock extends BaseElementRenderable implements
 		}
 	}
 
-	// public Iterator getExportedRenderables() {
-	// RBlockViewport bodyLayout = this.bodyLayout;
-	// return bodyLayout == null ? null : bodyLayout.getExportedRenderables();
-	// }
-	//
 	public RBlockViewport getRBlockViewport() {
 		return this.bodyLayout;
 	}
 
+	public boolean extractSelectionText(StringBuffer buffer, boolean inSelection, RenderableSpot startPoint, RenderableSpot endPoint) {
+		boolean result = super.extractSelectionText(buffer, inSelection, startPoint, endPoint);
+		String br = System.getProperty("line.separator");
+		if(inSelection) {
+			buffer.insert(0, br);
+		}
+		if(result) {
+			buffer.append(br);
+		}
+		return result;
+	}
+
 	public String toString() {
 		return "RBlock[node=" + this.modelNode + "]";
 	}
