@@ -81,6 +81,7 @@ public class RBlockViewport extends BaseRCollection {
 	private boolean skipParagraphBreakBefore;
 	//private boolean skipLineBreakBefore;
 	private static final Map elementLayout = new HashMap(70);
+	private static final MarkupLayout miscLayout = new MiscLayout();
 	
 	static {
 		Map el = elementLayout;
@@ -481,12 +482,10 @@ public class RBlockViewport extends BaseRCollection {
 					this.currentLine.addStyleChanger(new RStyleChanger(child));
 					String nodeName = child.getNodeName().toUpperCase();
 					MarkupLayout ml = (MarkupLayout) elementLayout.get(nodeName);
-					if(ml != null) {
-						ml.layoutMarkup(this, (HTMLElementImpl) child);
-					}
-					else {
-						this.layoutMarkup( child);
+					if(ml == null) {
+						ml = miscLayout;
 					}
+					ml.layoutMarkup(this, (HTMLElementImpl) child);
 					this.currentLine.addStyleChanger(new RStyleChanger(node));
 				}
 				else if(nodeType == Node.COMMENT_NODE || nodeType == Node.PROCESSING_INSTRUCTION_NODE) {
@@ -1720,6 +1719,12 @@ public class RBlockViewport extends BaseRCollection {
 		}
 	}
 
+	private static class MiscLayout extends CommonLayout {
+		public MiscLayout() {
+			super(DISPLAY_INLINE);
+		}
+	}
+
 	private static class HLayout extends CommonLayout {		
 		public HLayout(int fontSize) {
 			super(DISPLAY_BLOCK);
