@@ -114,6 +114,24 @@ class RLine extends BaseRCollection {
 		}
 	}
 		
+	public boolean extractSelectionText(StringBuffer buffer, boolean inSelection, RenderableSpot startPoint, RenderableSpot endPoint) {
+		boolean result = super.extractSelectionText(buffer, inSelection, startPoint, endPoint);
+		if(result) {
+			LineBreak br = this.lineBreak;
+			if(br != null) {
+				buffer.append(System.getProperty("line.separator"));
+			}
+			else {
+				ArrayList renderables = this.renderables;
+				int size = renderables.size();
+				if(size > 0 && !(renderables.get(size-1) instanceof RBlank)) {
+					buffer.append(" ");
+				}					
+			}
+		}
+		return result;
+	}
+
 	public final void addStyleChanger(RStyleChanger sc) {
 		this.renderables.add(sc);
 	}
