@@ -68,6 +68,10 @@ class RUIControl extends BaseElementRenderable implements RElement {
 		return this.widget.getVAlign();
 	}
 
+	public boolean hasBackground() {
+		return this.backgroundColor != null || this.backgroundImage != null || this.lastBackgroundImageUri != null;
+	}
+	
 	public final void paint(Graphics g) {
 		RenderState rs = this.modelNode.getRenderState();
 		if(rs != null && rs.getVisibility() != RenderState.VISIBILITY_VISIBLE) {
@@ -263,4 +267,9 @@ class RUIControl extends BaseElementRenderable implements RElement {
 	public Color getPaintedBackgroundColor() {
 		return this.container.getPaintedBackgroundColor();
 	}
+	
+	public Color getForegroundColor() {
+		RenderState rs = this.modelNode.getRenderState();
+		return rs == null ? null : rs.getColor();
+	}
 }
