@@ -65,6 +65,7 @@ final class RWord extends BaseBoundableRenderable {
 	    int ascentPlusLeading = this.ascentPlusLeading;
 		int height = this.height;
 		int textDecoration = rs.getTextDecorationMask();
+		int textTransform = rs.getTextTransform();
 		Color bkg = rs.getTextBackgroundColor();
 		if(bkg != null) {
 			Color oldColor = g.getColor();
@@ -75,7 +76,26 @@ final class RWord extends BaseBoundableRenderable {
 				g.setColor(oldColor);
 			}				
 		}
-		g.drawString(word, 0, ascentPlusLeading);
+		String string;
+		if(textTransform == RenderState.TEXTTRANSFORM_NONE) {
+			string = word;
+		}
+		else {
+			switch(textTransform) {
+			case RenderState.TEXTTRANSFORM_CAPITALIZE:
+				string = Character.toTitleCase(word.charAt(0)) + word.substring(1).toLowerCase();
+				break;
+			case RenderState.TEXTTRANSFORM_LOWERCASE:
+				string = word.toLowerCase();
+				break;
+			case RenderState.TEXTTRANSFORM_UPPERCASE:
+				string = word.toUpperCase();
+				break;
+			default:
+				string = word;
+			}
+		}		
+		g.drawString(string, 0, ascentPlusLeading);
 		int td = textDecoration;
 		if(td != 0) {
 			if((td & RenderState.MASK_TEXTDECORATION_UNDERLINE) != 0) {
