@@ -16,7 +16,7 @@ GNU LESSER GENERAL PUBLIC LICENSE
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 package org.lobobrowser.html.renderer;
 import java.awt.Color;
@@ -181,7 +181,7 @@ class TableMatrix {
 	
 	private static HtmlLength getWidthLength(HTMLElementImpl element, int availWidth) {
 		try {
-			CSS2PropertiesImpl props = element.getCurrentStyle();
+			AbstractCSS2Properties props = element.getCurrentStyle();
 			String widthText = props == null ? null : props.getWidth();
 			if(widthText == null) {
 				return new HtmlLength(element.getAttribute("width"));
@@ -196,7 +196,7 @@ class TableMatrix {
 
 	private static HtmlLength getHeightLength(HTMLElementImpl element, int availHeight) {
 		try {
-			CSS2PropertiesImpl props = element.getCurrentStyle();
+			AbstractCSS2Properties props = element.getCurrentStyle();
 			String heightText = props == null ? null : props.getHeight();
 			if(heightText == null) {
 				return new HtmlLength(element.getAttribute("height"));
@@ -229,7 +229,7 @@ class TableMatrix {
 		ArrayList rowElements = this.ROW_ELEMENTS;
 		ArrayList allCells = this.ALL_CELLS;
 		Map rowElementToRowArray = new HashMap(2);
-		ArrayList cellList = te.getDescendents(COLUMNS_FILTER);
+		ArrayList cellList = te.getDescendents(COLUMNS_FILTER, false);
 		ArrayList currentNullRow = null;
 		Iterator ci = cellList.iterator();
 		while(ci.hasNext()) {
@@ -669,53 +669,58 @@ class TableMatrix {
 			}
 			RTableCell ac = vc == null ? null : vc.getActualCell();
 			if(ac != null) {
-				int colSpan = ac.getColSpan();
-				if(colSpan > 1) {
-					int firstCol = ac.getVirtualColumn();
-					int cellExtras = (colSpan - 1) * (cellSpacingX + 2 * hasBorder);
-					int vcActualWidth = cellExtras;
-					for(int x = 0; x < colSpan; x++) {
-						vcActualWidth += columnSizes[firstCol + x].actualSize;
-					}
-					//TODO: better height possible
-					Dimension size = ac.doCellLayout(vcActualWidth, 0, false, false);
-					int vcRenderWidth = size.width;
-					
-					int denominator = (vcActualWidth - cellExtras);
-					int newTentativeCellWidth;
-					if(denominator > 0) {
-						newTentativeCellWidth = actualSize * (vcRenderWidth - cellExtras) / denominator;
-					}
-					else {
-						newTentativeCellWidth = (vcRenderWidth - cellExtras) / colSpan;
-					}
-					if(newTentativeCellWidth > colSize.layoutSize) {
-						colSize.layoutSize = newTentativeCellWidth;
-					}
-					int rowSpan = ac.getRowSpan();
-					int vch = (size.height - (rowSpan - 1) * (this.cellSpacingY + 2 * hasBorder)) / rowSpan;
-					for(int y = 0; y < rowSpan; y++) {
-						if(rowSizes[row + y].minSize < vch) {
-							rowSizes[row + y].minSize = vch;
+				if(ac.getVirtualRow() == row) {
+					// Only process actual cells with a row
+					// beginning at the current row being processed.
+					int colSpan = ac.getColSpan();
+					if(colSpan > 1) {
+						int firstCol = ac.getVirtualColumn();
+						int cellExtras = (colSpan - 1) * (cellSpacingX + 2 * hasBorder);
+						int vcActualWidth = cellExtras;
+						for(int x = 0; x < colSpan; x++) {
+							vcActualWidth += columnSizes[firstCol + x].actualSize;
+						}
+						//TODO: better height possible
+						Dimension size = ac.doCellLayout(vcActualWidth, 0, false);
+						int vcRenderWidth = size.width;
+
+						int denominator = (vcActualWidth - cellExtras);
+						int newTentativeCellWidth;
+						if(denominator > 0) {
+							newTentativeCellWidth = actualSize * (vcRenderWidth - cellExtras) / denominator;
+						}
+						else {
+							newTentativeCellWidth = (vcRenderWidth - cellExtras) / colSpan;
+						}
+						if(newTentativeCellWidth > colSize.layoutSize) {
+							colSize.layoutSize = newTentativeCellWidth;
+						}
+						int rowSpan = ac.getRowSpan();
+						int vch = (size.height - (rowSpan - 1) * (this.cellSpacingY + 2 * hasBorder)) / rowSpan;
+						for(int y = 0; y < rowSpan; y++) {
+							if(rowSizes[row + y].minSize < vch) {
+								rowSizes[row + y].minSize = vch;
+							}
 						}
 					}
-				}
-				else {
-					//TODO: better height possible
-					Dimension size = ac.doCellLayout(actualSize, 0, false, false);
-					if(size.width > colSize.layoutSize) {
-						colSize.layoutSize = size.width;
-					}
-					int rowSpan = ac.getRowSpan();
-					int vch = (size.height - (rowSpan - 1) * (this.cellSpacingY + 2 * hasBorder)) / rowSpan;
-					for(int y = 0; y < rowSpan; y++) {
-						if(rowSizes[row + y].minSize < vch) {
-							rowSizes[row + y].minSize = vch;
+					else {
+						//TODO: better height possible
+						Dimension size = ac.doCellLayout(actualSize, 0, false);
+						if(size.width > colSize.layoutSize) {
+							colSize.layoutSize = size.width;
+						}
+						int rowSpan = ac.getRowSpan();
+						int vch = (size.height - (rowSpan - 1) * (this.cellSpacingY + 2 * hasBorder)) / rowSpan;
+						for(int y = 0; y < rowSpan; y++) {
+							if(rowSizes[row + y].minSize < vch) {
+								rowSizes[row + y].minSize = vch;
+							}
 						}
 					}
 				}
 			}
-			row = (ac == null ? row + 1 : ac.getVirtualRow() + ac.getRowSpan());
+			//row = (ac == null ? row + 1 : ac.getVirtualRow() + ac.getRowSpan());
+			row++;
 		}
 	}
 	
@@ -918,7 +923,7 @@ class TableMatrix {
 				if(!prevOverrideNoWrap) {
 					state.overrideNoWrap = !widthDeclared;
 				}
-				size = cell.doCellLayout(cellsTotalWidth, 0, false, false);
+				size = cell.doCellLayout(cellsTotalWidth, 0, false);
 			} finally {
 				state.overrideNoWrap = prevOverrideNoWrap;
 			}
@@ -1226,25 +1231,21 @@ class TableMatrix {
 			else {
 				totalCellHeight = rowSizes[row].actualSize;
 			}
-			Dimension size = cell.doCellLayout(totalCellWidth, totalCellHeight, true, true);
+			Dimension size = cell.doCellLayout(totalCellWidth, totalCellHeight, true);
 			if(size.width > totalCellWidth) {
 				if(colSpan == 1) {
-					if(size.width > colSizes[col].actualSize) {
-						colSizes[col].actualSize = size.width;
-					}
+					colSizes[col].actualSize = size.width;
 				}
 				else {
-					//TODO: Adjustment for colSpan > 1.
+					colSizes[col].actualSize += (size.width - totalCellWidth);
 				}
 			}
 			if(size.height > totalCellHeight) {
 				if(rowSpan == 1) {
-					if(size.height > rowSizes[row].actualSize) {
-						rowSizes[row].actualSize = size.height;
-					}
+					rowSizes[row].actualSize = size.height;
 				}
 				else {
-					//TODO: Adjustment for rowSpan > 1.
+					rowSizes[row].actualSize += (size.height - totalCellHeight);
 				}
 			}			
 		}
