@@ -199,7 +199,9 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 			for(int i = 0; i < length; i++) {
 				String propertyName = styleDeclaration.item(i);
 				String propertyValue = styleDeclaration.getPropertyValue(propertyName);
-				this.setPropertyValueProcessed(propertyName.toLowerCase(), propertyValue, styleDeclaration);
+				String priority = styleDeclaration.getPropertyPriority(propertyName);
+				boolean important = priority != null && priority.length() != 0 && "important".equals(priority);
+				this.setPropertyValueProcessed(propertyName.toLowerCase(), propertyValue, styleDeclaration, important);
 			}
 		}
 	}
@@ -236,7 +238,8 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 				}
 			}
 			if(vm != null) {
-				return (String) vm.get(lowerCaseName);
+				Property p = (Property) vm.get(lowerCaseName);
+				return p == null ? null : p.value;
 			}
 		}		
 		return null;
@@ -254,34 +257,45 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 				vm = new HashMap(1);
 				this.valueMap = vm;
 			}
-			vm.put(lowerCaseName, value);
+			vm.put(lowerCaseName, new Property(value, true));
 		}
 	}
 
 	/**
 	 * Alternate method called to set property values from
 	 * CSS declarations.
+	 * <p>
+	 * This method checks importance of the value.
 	 * @param lowerCaseName The name of the property in lowercase.
 	 * @param value The property value.
 	 */
-	protected final void setPropertyValueLCAlt(String lowerCaseName, String value) {
+	protected final void setPropertyValueLCAlt(String lowerCaseName, String value, boolean important) {
 		Map vm = this.valueMap;
 		synchronized(this) {
 			if(vm == null) {
 				vm = new HashMap(1);
 				this.valueMap = vm;
 			}
-			vm.put(lowerCaseName, value);
+			else {
+				if(!important) {
+					Property oldProperty = (Property) vm.get(lowerCaseName);
+					if(oldProperty != null && oldProperty.important) {
+						// Ignore setting
+						return;
+					}
+				}
+			}
+			vm.put(lowerCaseName, new Property(value, important));
 		}
 	}
 
-	protected final void setPropertyValueProcessed(String lowerCaseName, String value, CSSStyleDeclaration declaration) {
+	protected final void setPropertyValueProcessed(String lowerCaseName, String value, CSSStyleDeclaration declaration, boolean important) {
 		SubPropertySetter setter = (SubPropertySetter) SUB_SETTERS.get(lowerCaseName);
 		if(setter != null) {
-			setter.changeValue(this, value, declaration);
+			setter.changeValue(this, value, declaration, important);
 		}
 		else {
-			this.setPropertyValueLCAlt(lowerCaseName, value);			
+			this.setPropertyValueLCAlt(lowerCaseName, value, important);			
 		}
 	}
 	
@@ -1520,16 +1534,20 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 	}
 	
 	private static interface SubPropertySetter {
-		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration);
+		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration, boolean important);
 	}
 	
 	private static class BorderSetter1 implements SubPropertySetter {
 		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration) {
-			properties.setPropertyValueLCAlt(BORDER, newValue);
-			properties.setPropertyValueProcessed(BORDER_TOP, newValue, declaration);
-			properties.setPropertyValueProcessed(BORDER_LEFT, newValue, declaration);
-			properties.setPropertyValueProcessed(BORDER_BOTTOM, newValue, declaration);
-			properties.setPropertyValueProcessed(BORDER_RIGHT, newValue, declaration);
+			this.changeValue(properties, newValue, declaration, true);
+		}
+
+		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration, boolean important) {
+			properties.setPropertyValueLCAlt(BORDER, newValue, important);
+			properties.setPropertyValueProcessed(BORDER_TOP, newValue, declaration, important);
+			properties.setPropertyValueProcessed(BORDER_LEFT, newValue, declaration, important);
+			properties.setPropertyValueProcessed(BORDER_BOTTOM, newValue, declaration, important);
+			properties.setPropertyValueProcessed(BORDER_RIGHT, newValue, declaration, important);
 		}
 	}
 
@@ -1539,9 +1557,13 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 		public BorderSetter2(String baseName) {
 			this.name = baseName;
 		}
-		
+
 		public void changeValue(AbstractCSS2Properties properties, String value, CSSStyleDeclaration declaration) {
-			properties.setPropertyValueLCAlt(this.name, value);
+			this.changeValue(properties, value, declaration, true);
+		}
+
+		public void changeValue(AbstractCSS2Properties properties, String value, CSSStyleDeclaration declaration, boolean important) {
+			properties.setPropertyValueLCAlt(this.name, value, important);
 			if(value != null && value.length() > 0) {
 				String[] array = HtmlValues.splitCssValue(value);
 				String color = null;
@@ -1561,13 +1583,13 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 				}
 				String name = this.name;
 				if(color != null) {
-					properties.setPropertyValueLCAlt(name + "-color", color);
+					properties.setPropertyValueLCAlt(name + "-color", color, important);
 				}
 				if(width != null) {
-					properties.setPropertyValueLCAlt(name + "-width", width);
+					properties.setPropertyValueLCAlt(name + "-width", width, important);
 				}
 				if(style != null) {
-					properties.setPropertyValueLCAlt(name + "-style", style);
+					properties.setPropertyValueLCAlt(name + "-style", style, important);
 				}
 			}
 		}
@@ -1583,9 +1605,13 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 			this.suffix = suffix;
 			this.property = property;
 		}
-		
+
 		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration) {
-			properties.setPropertyValueLCAlt(this.property, newValue);
+			this.changeValue(properties, newValue, declaration, true);
+		}
+
+		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration, boolean important) {
+			properties.setPropertyValueLCAlt(this.property, newValue, important);
 			if(newValue != null && newValue.length() > 0) {
 				String[] array = HtmlValues.splitCssValue(newValue);
 				int size = array.length;
@@ -1593,34 +1619,34 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 					String prefix = this.prefix;
 					String suffix = this.suffix;
 					String value = array[0];
-					properties.setPropertyValueLCAlt(prefix + "top" + suffix, value);
-					properties.setPropertyValueLCAlt(prefix + "right" + suffix, value);
-					properties.setPropertyValueLCAlt(prefix + "bottom" + suffix, value);
-					properties.setPropertyValueLCAlt(prefix + "left" + suffix, value);
+					properties.setPropertyValueLCAlt(prefix + "top" + suffix, value, important);
+					properties.setPropertyValueLCAlt(prefix + "right" + suffix, value, important);
+					properties.setPropertyValueLCAlt(prefix + "bottom" + suffix, value, important);
+					properties.setPropertyValueLCAlt(prefix + "left" + suffix, value, important);
 				}
 				else if(size >= 4) {
 					String prefix = this.prefix;
 					String suffix = this.suffix;
-					properties.setPropertyValueLCAlt(prefix + "top" + suffix, array[0]);
-					properties.setPropertyValueLCAlt(prefix + "right" + suffix, array[1]);
-					properties.setPropertyValueLCAlt(prefix + "bottom" + suffix, array[2]);
-					properties.setPropertyValueLCAlt(prefix + "left" + suffix, array[3]);				
+					properties.setPropertyValueLCAlt(prefix + "top" + suffix, array[0], important);
+					properties.setPropertyValueLCAlt(prefix + "right" + suffix, array[1], important);
+					properties.setPropertyValueLCAlt(prefix + "bottom" + suffix, array[2], important);
+					properties.setPropertyValueLCAlt(prefix + "left" + suffix, array[3], important);				
 				}
 				else if(size == 2) {
 					String prefix = this.prefix;
 					String suffix = this.suffix;
-					properties.setPropertyValueLCAlt(prefix + "top" + suffix, array[0]);
-					properties.setPropertyValueLCAlt(prefix + "right" + suffix, array[1]);
-					properties.setPropertyValueLCAlt(prefix + "bottom" + suffix, array[0]);
-					properties.setPropertyValueLCAlt(prefix + "left" + suffix, array[1]);								
+					properties.setPropertyValueLCAlt(prefix + "top" + suffix, array[0], important);
+					properties.setPropertyValueLCAlt(prefix + "right" + suffix, array[1], important);
+					properties.setPropertyValueLCAlt(prefix + "bottom" + suffix, array[0], important);
+					properties.setPropertyValueLCAlt(prefix + "left" + suffix, array[1], important);								
 				}
 				else if(size == 3) {
 					String prefix = this.prefix;
 					String suffix = this.suffix;
-					properties.setPropertyValueLCAlt(prefix + "top" + suffix, array[0]);
-					properties.setPropertyValueLCAlt(prefix + "right" + suffix, array[1]);
-					properties.setPropertyValueLCAlt(prefix + "bottom" + suffix, array[2]);
-					properties.setPropertyValueLCAlt(prefix + "left" + suffix, array[1]);												
+					properties.setPropertyValueLCAlt(prefix + "top" + suffix, array[0], important);
+					properties.setPropertyValueLCAlt(prefix + "right" + suffix, array[1], important);
+					properties.setPropertyValueLCAlt(prefix + "bottom" + suffix, array[2], important);
+					properties.setPropertyValueLCAlt(prefix + "left" + suffix, array[1], important);												
 				}
 			}
 		}
@@ -1628,6 +1654,10 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 	
 	private static class BackgroundImageSetter implements SubPropertySetter {
 		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration) {
+			this.changeValue(properties, newValue, declaration, true);
+		}
+
+		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration, boolean important) {
 			String baseHref = null;
 			String finalValue;
 			if(declaration != null) {
@@ -1671,13 +1701,17 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 					}
 				}
 			}
-			properties.setPropertyValueLCAlt(BACKGROUND_IMAGE, finalValue);
+			properties.setPropertyValueLCAlt(BACKGROUND_IMAGE, finalValue, important);
 		}
 	}
 	
 	private static class BackgroundSetter implements SubPropertySetter {
 		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration) {
-			properties.setPropertyValueLCAlt(BACKGROUND, newValue);
+			this.changeValue(properties, newValue, declaration, true);
+		}
+		
+		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration, boolean important) {
+			properties.setPropertyValueLCAlt(BACKGROUND, newValue, important);
 			if(newValue != null && newValue.length() > 0) {
 				String[] tokens = HtmlValues.splitCssValue(newValue);
 				boolean hasXPosition = false;
@@ -1709,16 +1743,16 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 					}
 				}
 				if(color != null) {
-					properties.setPropertyValueLCAlt(BACKGROUND_COLOR, color);
+					properties.setPropertyValueLCAlt(BACKGROUND_COLOR, color, important);
 				}
 				if(image != null) {
-					properties.setPropertyValueProcessed(BACKGROUND_IMAGE, image, declaration);
+					properties.setPropertyValueProcessed(BACKGROUND_IMAGE, image, declaration, important);
 				}
 				if(backgroundRepeat != null) {
-					properties.setPropertyValueLCAlt(BACKGROUND_REPEAT, backgroundRepeat);
+					properties.setPropertyValueLCAlt(BACKGROUND_REPEAT, backgroundRepeat, important);
 				}
 				if(position != null) {
-					properties.setPropertyValueLCAlt(BACKGROUND_POSITION, position);
+					properties.setPropertyValueLCAlt(BACKGROUND_POSITION, position, important);
 				}
 			}
 		}
@@ -1727,25 +1761,29 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 	
 	private static class FontSetter implements SubPropertySetter {
 		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration) {
-			properties.setPropertyValueLCAlt(FONT, newValue);
+			this.changeValue(properties, newValue, declaration, true);
+		}
+		
+		public void changeValue(AbstractCSS2Properties properties, String newValue, CSSStyleDeclaration declaration, boolean important) {
+			properties.setPropertyValueLCAlt(FONT, newValue, important);
 			if(newValue != null && newValue.length() > 0) {
 				String fontSpecTL = newValue.toLowerCase();
 				FontInfo fontInfo = (FontInfo) HtmlValues.SYSTEM_FONTS.get(fontSpecTL);
 				if(fontInfo != null) {
 					if(fontInfo.fontFamily != null) {
-						properties.setPropertyValueLCAlt(FONT_FAMILY, fontInfo.fontFamily);
+						properties.setPropertyValueLCAlt(FONT_FAMILY, fontInfo.fontFamily, important);
 					}
 					if(fontInfo.fontSize != null) {
-						properties.setPropertyValueLCAlt(FONT_SIZE, fontInfo.fontSize);					
+						properties.setPropertyValueLCAlt(FONT_SIZE, fontInfo.fontSize, important);					
 					}
 					if(fontInfo.fontStyle != null) {
-						properties.setPropertyValueLCAlt(FONT_STYLE, fontInfo.fontStyle);										
+						properties.setPropertyValueLCAlt(FONT_STYLE, fontInfo.fontStyle, important);										
 					}
 					if(fontInfo.fontVariant != null) {
-						properties.setPropertyValueLCAlt(FONT_VARIANT, fontInfo.fontVariant);															
+						properties.setPropertyValueLCAlt(FONT_VARIANT, fontInfo.fontVariant, important);															
 					}
 					if(fontInfo.fontWeight != null) {
-						properties.setPropertyValueLCAlt(FONT_WEIGHT, fontInfo.fontWeight);																				
+						properties.setPropertyValueLCAlt(FONT_WEIGHT, fontInfo.fontWeight, important);																				
 					}
 					return;
 				}
@@ -1756,15 +1794,15 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 				for(i = 0; i < length; i++) {
 					token = tokens[i];
 					if(HtmlValues.isFontStyle(token)) {
-						properties.setPropertyValueLCAlt(FONT_STYLE, token);										
+						properties.setPropertyValueLCAlt(FONT_STYLE, token, important);										
 						continue;
 					}
 					if(HtmlValues.isFontVariant(token)) {
-						properties.setPropertyValueLCAlt(FONT_VARIANT, token);										
+						properties.setPropertyValueLCAlt(FONT_VARIANT, token, important);										
 						continue;
 					}
 					if(HtmlValues.isFontWeight(token)) {
-						properties.setPropertyValueLCAlt(FONT_WEIGHT, token);										
+						properties.setPropertyValueLCAlt(FONT_WEIGHT, token, important);										
 						continue;
 					}
 					// Otherwise exit loop
@@ -1773,10 +1811,10 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 				if(token != null) {
 					int slashIdx = token.indexOf('/');
 					String fontSizeText = slashIdx == -1 ? token : token.substring(0, slashIdx);
-					properties.setPropertyValueLCAlt(FONT_SIZE, fontSizeText);
+					properties.setPropertyValueLCAlt(FONT_SIZE, fontSizeText, important);
 					String lineHeightText = slashIdx == -1 ? null : token.substring(slashIdx+1);
 					if(lineHeightText != null) {
-						properties.setPropertyValueLCAlt(LINE_HEIGHT, lineHeightText);
+						properties.setPropertyValueLCAlt(LINE_HEIGHT, lineHeightText, important);
 					}
 					if(++i < length) {
 						StringBuffer fontFamilyBuff = new StringBuffer();
@@ -1785,10 +1823,20 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 							fontFamilyBuff.append(token);
 							fontFamilyBuff.append(' ');
 						} while(++i < length);
-						properties.setPropertyValueLCAlt(FONT_FAMILY, fontFamilyBuff.toString());
+						properties.setPropertyValueLCAlt(FONT_FAMILY, fontFamilyBuff.toString(), important);
 					}
 				}
 			}
 		}
 	}
+	
+	private static class Property {
+		public final String value;
+		public final boolean important;
+		
+		public Property(final String value, final boolean important) {
+			this.value = value;
+			this.important = important;
+		}
+	}
 }
