@@ -27,6 +27,7 @@ import java.awt.FontMetrics;
 import java.awt.Insets;
 import java.awt.Toolkit;
 import java.awt.GraphicsEnvironment;
+import java.awt.font.*;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Map;
@@ -339,11 +340,27 @@ public class HtmlValues {
 	public static final float getFontSize(String spec,
 			RenderState parentRenderState) {
 		String specTL = spec.toLowerCase();
-		if (specTL.endsWith("pt") || specTL.endsWith("px")
+		if(specTL.endsWith("em")) {
+			Font font = parentRenderState.getFont();
+			String pxText = specTL.substring(0, specTL.length() - 2);
+			double value;
+			try {
+				value = Double.parseDouble(pxText);
+			} catch (NumberFormatException nfe) {
+				return DEFAULT_FONT_SIZE_INT;
+			}
+			return (int) Math.round(font.getSize() * value);
+		}
+		else if (specTL.endsWith("px") || specTL.endsWith("pt")
 				|| specTL.endsWith("cm") || specTL.endsWith("pc")
 				|| specTL.endsWith("cm") || specTL.endsWith("mm")
-				|| specTL.endsWith("em") || specTL.endsWith("ex")) {
-			return getPixelSize(spec, parentRenderState, DEFAULT_FONT_SIZE_INT);
+				|| specTL.endsWith("ex")) {
+			int pixelSize = getPixelSize(spec, parentRenderState, DEFAULT_FONT_SIZE_INT);
+			int dpi = GraphicsEnvironment.isHeadless() ? 72 : Toolkit
+					.getDefaultToolkit().getScreenResolution();
+			// Normally the factor below should be 72, but
+			// the font-size concept in HTML is handled differently.
+			return pixelSize * 96 / dpi;
 		} else if (specTL.endsWith("%")) {
 			String value = specTL.substring(0, specTL.length() - 1);
 			try {
@@ -375,7 +392,7 @@ public class HtmlValues {
 		} else if ("smaller".equals(specTL)) {
 			int parentFontSize = parentRenderState == null ? DEFAULT_FONT_SIZE_INT
 					: parentRenderState.getFont().getSize();
-			return parentFontSize * 1.2f;
+			return parentFontSize / 1.2f;
 		} else {
 			return getPixelSize(spec, parentRenderState, DEFAULT_FONT_SIZE_INT);
 		}
@@ -399,9 +416,35 @@ public class HtmlValues {
 	public static final int getPixelSize(String spec, RenderState renderState,
 			int errorValue) {
 		String lcSpec = spec.toLowerCase();
-		if (lcSpec.endsWith("pt")) {
-			String valText = lcSpec.substring(0, lcSpec.length()
-					- "pt".length());
+		if (lcSpec.endsWith("px")) {
+			String pxText = lcSpec
+			.substring(0, lcSpec.length() - 2);
+			try {
+				return (int) Math.round(Double.parseDouble(pxText));
+			} catch (NumberFormatException nfe) {
+				return errorValue;
+			}
+		}
+		else if (lcSpec.endsWith("em") && renderState != null) {
+			Font f = renderState.getFont();
+			String valText = lcSpec.substring(0, lcSpec.length() - 2);
+			double val;
+			try {
+				val = Double.parseDouble(valText);
+			} catch (NumberFormatException nfe) {
+				return errorValue;
+			}
+			// Get fontSize in 1/72 of an inch.
+			int fontSize = f.getSize();
+			int dpi = GraphicsEnvironment.isHeadless() ? 72 : Toolkit
+					.getDefaultToolkit().getScreenResolution();
+			// The factor below should normally be 72, but font sizes
+			// are calculated differently in HTML.
+			double pixelSize = fontSize * dpi / 96;
+			return (int) Math.round(pixelSize * val);
+		}
+		else if (lcSpec.endsWith("pt")) {
+			String valText = lcSpec.substring(0, lcSpec.length() - 2);
 			double val;
 			try {
 				val = Double.parseDouble(valText);
@@ -412,17 +455,8 @@ public class HtmlValues {
 					.getDefaultToolkit().getScreenResolution();
 			double inches = val / 72;
 			return (int) Math.round(dpi * inches);
-		} else if (lcSpec.endsWith("px")) {
-			String pxText = lcSpec
-					.substring(0, lcSpec.length() - "px".length());
-			try {
-				return (int) Math.round(Double.parseDouble(pxText));
-			} catch (NumberFormatException nfe) {
-				return errorValue;
-			}
 		} else if (lcSpec.endsWith("pc")) {
-			String valText = lcSpec.substring(0, lcSpec.length()
-					- "pc".length());
+			String valText = lcSpec.substring(0, lcSpec.length() - 2);
 			double val;
 			try {
 				val = Double.parseDouble(valText);
@@ -433,8 +467,7 @@ public class HtmlValues {
 			double inches = val / 6;
 			return (int) Math.round(dpi * inches);
 		} else if (lcSpec.endsWith("cm")) {
-			String valText = lcSpec.substring(0, lcSpec.length()
-					- "cm".length());
+			String valText = lcSpec.substring(0, lcSpec.length() - 2);
 			double val;
 			try {
 				val = Double.parseDouble(valText);
@@ -445,8 +478,7 @@ public class HtmlValues {
 			double inches = val / 2.54;
 			return (int) Math.round(dpi * inches);
 		} else if (lcSpec.endsWith("mm")) {
-			String valText = lcSpec.substring(0, lcSpec.length()
-					- "mm".length());
+			String valText = lcSpec.substring(0, lcSpec.length() - 2);
 			double val;
 			try {
 				val = Double.parseDouble(valText);
@@ -457,27 +489,16 @@ public class HtmlValues {
 			double inches = val / 25.4;
 			return (int) Math.round(dpi * inches);
 		} else if (lcSpec.endsWith("ex") && renderState != null) {
-			FontMetrics fm = renderState.getFontMetrics();
-			String valText = lcSpec.substring(0, lcSpec.length()
-					- "ex".length());
+			// Factor below is to try to match size in other browsers.
+			double xHeight = renderState.getFontMetrics().getAscent() * 0.47;
+			String valText = lcSpec.substring(0, lcSpec.length() - 2);
 			double val;
 			try {
 				val = Double.parseDouble(valText);
 			} catch (NumberFormatException nfe) {
 				return errorValue;
 			}
-			return (int) Math.round(fm.getAscent() * val);
-		} else if (lcSpec.endsWith("em") && renderState != null) {
-			Font f = renderState.getFont();
-			String valText = lcSpec.substring(0, lcSpec.length()
-					- "ex".length());
-			double val;
-			try {
-				val = Double.parseDouble(valText);
-			} catch (NumberFormatException nfe) {
-				return errorValue;
-			}
-			return (int) Math.round(f.getSize() * val);
+			return (int) Math.round(xHeight * val);
 		} else {
 			String pxText = lcSpec;
 			try {
