@@ -32,6 +32,13 @@ public interface RenderState {
 	public static final int MASK_TEXTDECORATION_OVERLINE = 2;
 	public static final int MASK_TEXTDECORATION_LINE_THROUGH = 4;
 	public static final int MASK_TEXTDECORATION_BLINK = 8;
+
+	public static final int TEXTTRANSFORM_NONE = 0;
+	public static final int TEXTTRANSFORM_CAPITALIZE = 1;
+	public static final int TEXTTRANSFORM_UPPERCASE = 2;
+	public static final int TEXTTRANSFORM_LOWERCASE = 4;
+//	TODO how to handle style cascading?	
+//	public static final int TEXTTRANSFORM_INHERIT = 8;
 	
 	public static final int DISPLAY_NONE = 0;
 	public static final int DISPLAY_INLINE = 1;
@@ -69,6 +76,7 @@ public interface RenderState {
 	public Color getTextBackgroundColor();
 	public BackgroundInfo getBackgroundInfo();
 	public Color getOverlayColor();
+	public int getTextTransform();
 	public int getTextDecorationMask();
 	public FontMetrics getFontMetrics();
 	public int getBlankWidth();
