@@ -77,6 +77,10 @@ public abstract class RenderStateDelegator implements RenderState {
 	public int getTextDecorationMask() {
 		return delegate.getTextDecorationMask();
 	}
+	
+	public int getTextTransform() {
+		return delegate.getTextTransform();
+	}
 
 	public WordInfo getWordInfo(String word) {
 		return delegate.getWordInfo(word);
