@@ -24,6 +24,7 @@ package org.lobobrowser.html.style;
 import java.net.MalformedURLException;
 import java.util.*;
 
+import org.lobobrowser.html.*;
 import org.lobobrowser.html.domimpl.HTMLDocumentImpl;
 import org.lobobrowser.html.domimpl.HTMLElementImpl;
 import org.w3c.dom.css.*;
@@ -116,14 +117,17 @@ public class StyleSheetAggregator {
 			//TODO: Other types of selectors
 		}
 		else if(rule instanceof CSSImportRule) {
-			CSSImportRule importRule = (CSSImportRule) rule;
-			if(CSSUtilities.matchesMedia(importRule.getMedia(), document.getUserAgentContext())) {
-				String href = importRule.getHref();
-				String styleHref = styleSheet.getHref();
-				String baseHref = styleHref == null ? document.getBaseURI() : styleHref;
-				CSSStyleSheet sheet = CSSUtilities.parse(styleSheet.getOwnerNode(), href, document, baseHref, false);
-				if(sheet != null) {
-					this.addStyleSheet(sheet);
+			UserAgentContext uacontext = document.getUserAgentContext();
+			if(uacontext.isExternalCSSEnabled()) {
+				CSSImportRule importRule = (CSSImportRule) rule;
+				if(CSSUtilities.matchesMedia(importRule.getMedia(), uacontext)) {
+					String href = importRule.getHref();
+					String styleHref = styleSheet.getHref();
+					String baseHref = styleHref == null ? document.getBaseURI() : styleHref;
+					CSSStyleSheet sheet = CSSUtilities.parse(styleSheet.getOwnerNode(), href, document, baseHref, false);
+					if(sheet != null) {
+						this.addStyleSheet(sheet);
+					}
 				}
 			}
 		}
