@@ -36,8 +36,10 @@ import org.w3c.dom.html2.*;
  */
 public class StyleSheetRenderState implements RenderState {
 	private static final FontFactory FONT_FACTORY = FontFactory.getInstance();
-	private static final String DEFAULT_FONT_FAMILY = "Times New Roman";
-	private static final Font DEFAULT_FONT = FONT_FACTORY.getFont(DEFAULT_FONT_FAMILY, null, null, null, HtmlValues.DEFAULT_FONT_SIZE);
+	// Default font needs to be something that displays in all languages.
+	// Serif, SansSerif, Monospaced.
+	private static final String DEFAULT_FONT_FAMILY = "SansSerif"; 
+	private static final Font DEFAULT_FONT = FONT_FACTORY.getFont(DEFAULT_FONT_FAMILY, null, null, null, HtmlValues.DEFAULT_FONT_SIZE, null);
 	protected static final Insets INVALID_INSETS = new Insets(0, 0, 0, 0);
 	protected static final BackgroundInfo INVALID_BACKGROUND_INFO = new BackgroundInfo();
 	protected static final Color INVALID_COLOR = new Color(100, 0, 100);
@@ -54,6 +56,7 @@ public class StyleSheetRenderState implements RenderState {
 	private Color iOverlayColor = INVALID_COLOR;
 	private BackgroundInfo iBackgroundInfo = INVALID_BACKGROUND_INFO;
 	private int iTextDecoration = -1;
+	private int iTextTransform = -1;
 	private int iBlankWidth = -1;
 	private boolean iHighlight;
 
@@ -301,7 +304,9 @@ public class StyleSheetRenderState implements RenderState {
 				fontWeight = "bold";
 			}			
 		}
-		f = FONT_FACTORY.getFont(fontFamily, fontStyle, fontVariant, fontWeight, fontSize.floatValue());
+		HTMLDocumentImpl document = this.document;
+		Set locales = document == null ? null : document.getLocales();
+		f = FONT_FACTORY.getFont(fontFamily, fontStyle, fontVariant, fontWeight, fontSize.floatValue(), locales);
 		this.iFont = f;
 		return f;
 	}
@@ -442,6 +447,48 @@ public class StyleSheetRenderState implements RenderState {
 		this.iTextDecoration = td;
 		return td;
 	}
+
+	public int getTextTransform()
+	{
+		int tt = this.iTextTransform;
+		if(tt != -1) {
+			return tt;
+		}
+		AbstractCSS2Properties props = this.getCssProperties();
+		String tdText = props == null ? null : props.getTextTransform();
+		if(tdText == null) {
+			RenderState prs = this.prevRenderState;
+			if(prs != null) {
+				tt = prs.getTextTransform();
+				this.iTextTransform = tt;
+				return tt;
+			}
+		}
+		tt = 0;
+		if(tdText != null) {
+			if("none".equals(tdText)) {
+				// continue
+			}
+			else if("capitalize".equals(tdText)) {				
+				tt = TEXTTRANSFORM_CAPITALIZE;
+			}
+			else if("uppercase".equals(tdText)) {
+				tt = TEXTTRANSFORM_UPPERCASE;
+			}
+			else if("lowercase".equals(tdText)) {
+				tt = TEXTTRANSFORM_LOWERCASE;
+			}
+// TODO how the explicit "inherit" value is to be handled?
+// 		Who is responsible for CSS cascading? 
+//			... painting code? prevRenderState?
+//			
+//			else if("inherit".equals(tdText)) {
+//				tt = TEXTTRANSFORM_INHERIT;								
+//			}
+		}
+		this.iTextTransform = tt;
+		return tt;
+	}	
 	
 	public final FontMetrics getFontMetrics() {
 		FontMetrics fm = this.iFontMetrics;
@@ -786,6 +833,16 @@ public class StyleSheetRenderState implements RenderState {
 			binfo.backgroundXPositionAbsolute = false;
 			binfo.backgroundXPosition = 0;			
 		}
+		else if("bottom".equalsIgnoreCase(xposition)) {
+			// Can happen
+			binfo.backgroundYPositionAbsolute = false;
+			binfo.backgroundYPosition = 100;			
+		}
+		else if("top".equalsIgnoreCase(xposition)) {
+			// Can happen
+			binfo.backgroundYPositionAbsolute = false;
+			binfo.backgroundYPosition = 0;			
+		}
 		else {
 			binfo.backgroundXPositionAbsolute = true;
 			binfo.backgroundXPosition = HtmlValues.getPixelSize(xposition, this, 0);
@@ -813,6 +870,16 @@ public class StyleSheetRenderState implements RenderState {
 			binfo.backgroundYPositionAbsolute = false;
 			binfo.backgroundYPosition = 0;			
 		}
+		else if("right".equalsIgnoreCase(yposition)) {
+			// Can happen
+			binfo.backgroundXPositionAbsolute = false;
+			binfo.backgroundXPosition = 100;			
+		}
+		else if("left".equalsIgnoreCase(yposition)) {
+			// Can happen
+			binfo.backgroundXPositionAbsolute = false;
+			binfo.backgroundXPosition = 0;			
+		}
 		else {
 			binfo.backgroundYPositionAbsolute = true;
 			binfo.backgroundYPosition = HtmlValues.getPixelSize(yposition, this, 0);
@@ -820,6 +887,10 @@ public class StyleSheetRenderState implements RenderState {
 	}
 	
 	private void applyBackgroundPosition(BackgroundInfo binfo, String position) {
+		binfo.backgroundXPositionAbsolute = false;
+		binfo.backgroundYPositionAbsolute = false;
+		binfo.backgroundXPosition = 50;
+		binfo.backgroundYPosition = 50;
 		StringTokenizer tok = new StringTokenizer(position, " \t\r\n");
 		if(tok.hasMoreTokens()) {
 			String xposition = tok.nextToken();
