@@ -26,7 +26,6 @@ package org.lobobrowser.html.test;
 import java.awt.event.MouseEvent;
 import java.io.*;
 import java.net.*;
-import java.util.*;
 
 import javax.swing.JOptionPane;
 
@@ -279,7 +278,9 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 		URL resolvedURL;
 		if("GET".equals(actualMethod) && formInputs != null) {
 			boolean firstParam = true;
-			StringBuffer newUrlBuffer = new StringBuffer(action.toExternalForm());
+			//TODO: What about the userInfo part of the URL?
+			URL noRefAction = new URL(action.getProtocol(), action.getHost(), action.getPort(), action.getFile());
+			StringBuffer newUrlBuffer = new StringBuffer(noRefAction.toExternalForm());
 			if(action.getQuery() == null) {
 				newUrlBuffer.append("?");
 			}
@@ -313,7 +314,7 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 			resolvedURL = action;
 		}
 		URL urlForLoading;
-		if(resolvedURL.getProtocol().equals("file")) {
+		if(resolvedURL.getProtocol().equalsIgnoreCase("file")) {
 			// Remove query so it works.
 			try {
 				String ref = action.getRef();
@@ -687,8 +688,8 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 	/**
 	 * This method must be overridden to implement a context menu.
 	 */
-	public void onContextMenu(HTMLElement element, MouseEvent event) {
-		this.warn("onContextMenu(): Not overridden.");
+	public boolean onContextMenu(HTMLElement element, MouseEvent event) {
+		return true;
 	}
 
 	/**
@@ -704,4 +705,16 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 	 */
 	public void onMouseOver(HTMLElement element, MouseEvent event) {
 	}
+
+	public boolean isImageLoadingEnabled() {
+		return true;
+	}
+
+	public boolean onDoubleClick(HTMLElement element, MouseEvent event) {
+		return true;
+	}
+
+	public boolean onMouseClick(HTMLElement element, MouseEvent event) {
+		return true;
+	}
 }
