@@ -48,10 +48,11 @@ public class FontFactory {
 		Set fontFamilies = this.fontFamilies;
 		synchronized(this) {
 			for(int i = 0; i < ffns.length; i++) {
+				String ffn = ffns[i];
 				if(liflag) {
-					logger.fine("FontFactory(): family=" + ffns[i]);
+					logger.fine("FontFactory(): family=" + ffn);
 				}
-				fontFamilies.add(ffns[i].toLowerCase());
+				fontFamilies.add(ffn.toLowerCase());
 			}
 		}
 	}
@@ -86,8 +87,8 @@ public class FontFactory {
 		}
 	}
 	
-	public Font getFont(String fontFamily, String fontStyle, String fontVariant, String fontWeight, float fontSize) {
-		FontKey key = new FontKey(fontFamily, fontStyle, fontVariant, fontWeight, fontSize);
+	public Font getFont(String fontFamily, String fontStyle, String fontVariant, String fontWeight, float fontSize, Set locales) {
+		FontKey key = new FontKey(fontFamily, fontStyle, fontVariant, fontWeight, fontSize, locales);
 		synchronized(this) {
 			Font font = (Font) this.fontMap.get(key);
 			if(font == null) {
@@ -98,22 +99,44 @@ public class FontFactory {
 		}
 	}
 	
+	private String defaultFontName = "SansSerif";
+	
+	public String getDefaultFontName() {
+		return defaultFontName;
+	}
+
+	/**
+	 * Sets the default font name to be used when a name is unrecognized
+	 * or when a font is determined not to be capable of diplaying characters
+	 * from a given language.
+	 * This should be the name of a font that can display unicode text
+	 * across all or most languages.
+	 * @param defaultFontName The name of a font.
+	 */
+	public void setDefaultFontName(String defaultFontName) {
+		if(defaultFontName == null) {
+			throw new IllegalArgumentException("defaultFontName cannot be null");
+		}
+		this.defaultFontName = defaultFontName;
+	}
+
 	private final Font createFont(FontKey key) {
 		String fontNames = key.fontFamily;
-		String fontFam = null;
+		String matchingFace = null;
 		Set fontFamilies = this.fontFamilies;
 		Map registeredFonts = this.registeredFonts; 
 		Font baseFont = null;
 		if(fontNames != null) {
 			StringTokenizer tok = new StringTokenizer(fontNames, ",");
 			while(tok.hasMoreTokens()) {
-				fontFam = tok.nextToken().trim();
-				String fontFamTL = fontFam.toLowerCase();
-				if(registeredFonts.containsKey(fontFamTL)) {
-					baseFont = (Font) registeredFonts.get(fontFamTL);
+				String face = tok.nextToken().trim();
+				String faceTL = face.toLowerCase();
+				if(registeredFonts.containsKey(faceTL)) {
+					baseFont = (Font) registeredFonts.get(faceTL);
 					break;
 				}
-				else if(fontFamilies.contains(fontFamTL)) {
+				else if(fontFamilies.contains(faceTL)) {
+					matchingFace = faceTL;
 					break;
 				}
 			}
@@ -128,9 +151,33 @@ public class FontFactory {
 		if(baseFont != null) {
 			return baseFont.deriveFont(fontStyle, key.fontSize);
 		}
-		else {
-			return new Font(fontFam, fontStyle,(int) Math.round(key.fontSize));
-		}
+		else if(matchingFace != null) {
+			Font font = new Font(matchingFace, fontStyle, (int) Math.round(key.fontSize));
+			Set locales = key.locales;
+			if(locales == null) {
+				Locale locale = Locale.getDefault();
+				if(font.canDisplayUpTo(locale.getDisplayLanguage(locale)) == -1) {
+					return font;
+				}				
+			}
+			else {
+				Iterator i = locales.iterator();
+				boolean allMatch = true;
+				while(i.hasNext()) {
+					Locale locale = (Locale) i.next();
+					if(font.canDisplayUpTo(locale.getDisplayLanguage(locale)) != -1) {
+						allMatch = false;
+						break;
+					}				
+				}
+				if(allMatch) {
+					return font;
+				}
+			}
+			// Otherwise, fall through.
+		}		
+		// Last resort:
+		return new Font(this.defaultFontName, fontStyle, (int) Math.round(key.fontSize));
 	}
 	
 	private static class FontKey {
@@ -139,6 +186,7 @@ public class FontFactory {
 		public final String fontVariant; 
 		public final String fontWeight; 
 		public final float fontSize;
+		public final Set locales;
 		
 		
 		/**
@@ -150,28 +198,45 @@ public class FontFactory {
 		 */
 		public FontKey(final String fontFamily, final String fontStyle,
 				final String fontVariant, final String fontWeight,
-				final float fontSize) {
-			this.fontFamily = fontFamily;
-			this.fontStyle = fontStyle;
-			this.fontVariant = fontVariant;
-			this.fontWeight = fontWeight;
+				final float fontSize, final Set locales) {
+			this.fontFamily = fontFamily == null ? null : fontFamily.intern();
+			this.fontStyle = fontStyle == null ? null : fontStyle.intern();
+			this.fontVariant = fontVariant == null ? null : fontVariant.intern();
+			this.fontWeight = fontWeight == null ? null : fontWeight.intern();
 			this.fontSize = fontSize;
+			this.locales = locales;
 		}
 		
 		public boolean equals(Object other) {
-			if(!(other instanceof FontKey)) {
+			if(other == this) {
+				// Quick check.
+				return true;
+			}
+			FontKey ors;
+			try {
+				ors = (FontKey) other;
+			} catch(ClassCastException cce) {
+				// Not expected
 				return false;
 			}
-			FontKey ors = (FontKey) other;
-			// fontSize is primitive
+			// Note that we use String.intern() for all string fields,
+			// so we can do instance comparisons.
 			return this.fontSize == ors.fontSize &&
-				   Objects.equals(this.fontFamily,ors.fontFamily) &&
-				   Objects.equals(this.fontStyle,ors.fontStyle) &&
-				   Objects.equals(this.fontWeight,ors.fontWeight) &&
-				   Objects.equals(this.fontVariant,ors.fontVariant);
+				   this.fontFamily == ors.fontFamily &&
+				   this.fontStyle == ors.fontStyle &&
+				   this.fontWeight == ors.fontWeight &&
+				   this.fontVariant == ors.fontVariant &&
+				   Objects.equals(this.locales,ors.locales);
 		}
+
+		private int cachedHash = -1;
 		
 		public int hashCode() {
+			int ch = this.cachedHash;
+			if(ch != -1) {
+				// Object is immutable - caching is ok.
+				return ch;
+			}
 			String ff = this.fontFamily;
 			if(ff == null) {
 				ff = "";
@@ -184,10 +249,12 @@ public class FontFactory {
 			if(fs == null) {
 				fs = "";
 			}
-			return ff.hashCode() ^ 
+			ch = ff.hashCode() ^ 
 				   fw.hashCode() ^ 
 				   fs.hashCode() ^
 				   (int) this.fontSize;
+			this.cachedHash = ch;
+			return ch;
 		}
 		
 		public String toString() {
